# How to contribute

## Preparing the development environment

Make sure you have development dependencies installed in your development environment.

```
pip install -r requirements-dev.txt
```

## pre-commit hooks

We use [pre-commit](https://pre-commit.com/) to maintain and install pre-commit
hooks that should be executed before each commit.


Please install these required tools on your system:

* [hadolint](https://github.com/hadolint/hadolint) for linting the `Dockerfile`
* [shellcheck](https://www.shellcheck.net/) and
* [shfmt](https://github.com/mvdan/sh) to lint and validate shell scripts

Then install pre-commit hooks:

```
pre-commit install
```

Running `git commmit` for the first time after installing the hook usually takes a
little longer because `pre-commit` pulls it's hooks from upstream repositories.
You can find all hooks in `.pre-commit-config.yaml`.

In case you need to skip the execution of the pre-commit hooks (please don't!),
use `git commit --no-verify`.


## Adding dependencies

Make sure you update our `requirements.txt` file before you push your changes.
Whenever you need a new python package, add it to requirements.in and run

`pip-compile`

to generate an new `requirements.txt` which does not only pin the new package
but also its dependencies.

If the new package you are adding is only used by developers,
please add it to the `requirements-dev.txt` file.
